<?php
//
// NCPA Server Config Wizard
// Copyright (c) 2014-2015 Nagios Enterprises, LLC. All rights reserved.
//

include_once(dirname(__FILE__) . '/../configwizardhelper.inc.php');

ncpa_configwizard_init();

function ncpa_configwizard_init()
{
    $name = "ncpa";
    $args = array(
        CONFIGWIZARD_NAME => $name,
        CONFIGWIZARD_VERSION => "1.3.3",
        CONFIGWIZARD_TYPE => CONFIGWIZARD_TYPE_MONITORING,
        CONFIGWIZARD_DESCRIPTION => _("Monitor an NCPA Agent"),
        CONFIGWIZARD_DISPLAYTITLE => _("NCPA Agent"),
        CONFIGWIZARD_FUNCTION => "ncpa_configwizard_func",
        CONFIGWIZARD_PREVIEWIMAGE => "ncpa.png",
        CONFIGWIZARD_FILTER_GROUPS => array('nagios','windows','linux','otheros'),
        CONFIGWIZARD_REQUIRES_VERSION => 500
    );
    register_configwizard($name, $args);
}


/**
 * @param string $mode
 * @param null   $inargs
 * @param        $outargs
 * @param        $result
 *
 * @return string
 */
function ncpa_configwizard_func($mode = "", $inargs = null, &$outargs, &$result)
{
    $wizard_name = "ncpa";

    // Initialize return code and output
    $result = 0;
    $output = "";

    // Initialize output args - pass back the same data we got
    $outargs[CONFIGWIZARD_PASSBACK_DATA] = $inargs;

    switch ($mode) {

        case CONFIGWIZARD_MODE_GETSTAGE1HTML:
            $address = grab_array_var($inargs, "address", "");
            $port = grab_array_var($inargs, "port", "5693");
            $token = grab_array_var($inargs, "token", "");

            $output = '
            <h5 class="ul">' . _('NCPA Agent') . '</h5>
            <p>' . _('Specify the connection details of the NCPA agent') . '.</p>
            <table class="table table-condensed table-no-border table-auto-width table-padded">
                <tr>
                    <td class="vt"><label>' . _('Address') . ':</label></td>
                    <td>
                        <input type="text" size="40" name="address" id="address" value="' . encode_form_val($address) . '" class="textfield form-control">
                        <div class="subtext">' . _('The IP address or FQDNS name of the NCPA Agent') . '.</div>
                    </td>
                </tr>
                <tr>
                    <td class="vt"><label>' . _('Port') . ':</label></td>
                    <td>
                        <input type="text" size="5" name="port" id="port" value="' . encode_form_val($port) . '" class="textfield form-control">
                        <div class="subtext">' . _('Defaults to port 5693') . '.</div>
                    </td>
                </tr>
                <tr>
                    <td class="vt"><label>' . _('Token') . ':</label></td>
                    <td>
                        <input type="text" size="20" name="token" id="token" value="' . encode_form_val($token) . '" class="textfield form-control">
                        <div class="subtext">' . _('Authentication token used to connect to the NCPA Agent') . '.</div>
                    </td>
                </tr>
            </table>';
            break;

        case CONFIGWIZARD_MODE_VALIDATESTAGE1DATA:
            // Get variables that were passed to us
            $address = grab_array_var($inargs, "address", "");
            $port = grab_array_var($inargs, "port", "5693");
            $token = grab_array_var($inargs, "token", "");

            // Check for errors
            $errors = 0;
            $errmsg = array();

            if (have_value($address) == false) {
                $errmsg[$errors++] = "No address specified.";
            }
            if (have_value($port) == false) {
                $errmsg[$errors++] = "No port number specified.";
            }
            if (have_value($token) == false) {
                $errmsg[$errors++] = "No token specified.";
            }

            // The URL we will use to query the NCPA agent, and do a walk
            // of all monitorable items.
            $query_url = "https://{$address}:{$port}/testconnect?token={$token}";

            // All we want to do is test if we can hit this URL.
            $raw_json = file_get_contents($query_url, 0, null, null);
            if (empty($raw_json)) {
                $errmsg[$errors++] = "Unable to contact server at {$query_url}.";
            } else {
                $json = json_decode($raw_json, true);
                if (!array_key_exists('value', $json)) {
                    $errmsg[$errors++] = "Bad token for connection.";
                }
            }
            if ($errors > 0) {
                $outargs[CONFIGWIZARD_ERROR_MESSAGES] = $errmsg;
                $result = 1;
            }
            break;

        case CONFIGWIZARD_MODE_GETSTAGE2HTML:
            // Get variables that were passed to us
            $address = grab_array_var($inargs, "address", "");
            $port = grab_array_var($inargs, "port", "");
            $token = grab_array_var($inargs, "token", "");
            $hostname = grab_array_var($inargs, "hostname", $address);
            $services_serial = grab_array_var($inargs, "services_serial", "");
            if ($services_serial) {
                $services = unserialize(base64_decode($services_serial));
            }

            // The URL we will use to query the NCPA agent, and do a walk
            // of all monitorable items. Make three queries, one for disk,
            // one for interfaces, and one for services.
            $iface_url = "https://{$address}:{$port}/api/interface?token={$token}";
            $disks_url = "https://{$address}:{$port}/api/disk?token={$token}";
            $services_api_url = "https://{$address}:{$port}/api/service?token={$token}";

            $iface_json = file_get_contents($iface_url, 0, null, null);
            $iface_data = json_decode($iface_json, true);
            $iface_root = $iface_data['value']['interface'];

            $disks_json = file_get_contents($disks_url, 0, null, null);
            $disks_data = json_decode($disks_json, true);
            $disks_root = $disks_data['value']['disk'];

            $services_api_json = file_get_contents($services_api_url, 0, null, null);
            $services_api_data = json_decode($services_api_json, true);
            $services_api_root = $services_api_data['value']['service'];
            if (empty($services_api_root)) {
                $services_api_root = $services_api_data['value']['services'];
            }

            $categories = array();
            $root = array();

            $root['disk'] = $disks_root;
            $root['interface'] = $iface_root;
            $root['services'] = $services_api_root;

            $output = '
            <input type="hidden" name="address" value="' . encode_form_val($address) . '">
            <input type="hidden" name="port" value="' . encode_form_val($port) . '">
            <input type="hidden" name="token" value="' . encode_form_val($token) . '">

            <h5 class="ul">' . _("NCPA Agent Information") . '</h5>
            <table class="table table-condensed table-no-border table-auto-width table-padded">
                <tr>
                    <td class="vt"><label>' . _("Address") . ':</label></td>
                    <td>
                        <input type="text" size="20" value="' . encode_form_val($address) . '" class="textfield form-control" disabled>
                    </td>
                </tr>
                <tr>
                    <td class="vt"><label>' . _("Host Name") . ':</label></td>
                    <td>
                        <input type="text" size="20" name="hostname" id="hostname" value="' . encode_form_val($hostname) . '" class="textfield form-control">
                        <div class="subtext">' . _("The name you'd like to have associated with this NCPA Agent") . '.</div>
                    </td>
                </tr>
                <tr>
                    <td class="vt"><label>' . _("Port") . ':</label></td>
                    <td>
                        <input type="text" size="5" value="' . encode_form_val($port) . '" class="textfield form-control" disabled>
                    </td>
                </tr>
                <tr>
                    <td><label>' . _("Token") . ':</label></td>
                    <td>
                        <input type="text" size="20" value="' . encode_form_val($token) . '" class="textfield form-control" disabled>
                    </td>
                </tr>
            </table>';

            // Set defaults for services
            $default_services['cpu_usage']['monitor'] = 'on';
            $default_services['cpu_usage']['warning'] = 20;
            $default_services['cpu_usage']['critical'] = 40;
            $default_services['memory_usage']['monitor'] = 'on';
            $default_services['memory_usage']['warning'] = 50;
            $default_services['memory_usage']['critical'] = 80;
            $default_services['swap_usage']['monitor'] = 'on';
            $default_services['swap_usage']['warning'] = 50;
            $default_services['swap_usage']['critical'] = 80;

            foreach ($root['disk']['logical'] as $title => $value) {
                $default_services['disk'][$title]['monitor'] = 'on';
                $default_services['disk'][$title]['warning'] = 70;
                $default_services['disk'][$title]['critical'] = 90;
                $default_services['disk'][$title]['name'] = str_replace("|", "/", $title);
            }

            ksort($root['interface']);
            foreach ($root['interface'] as $title => $value) {
                if (stripos($title, "Local Area Connection") !== false || stripos($title, "eth") !== false || stripos($title, "Wireless") !== false) {
                    $default_services['interface'][$title]['monitor'] = 'on';
                } else {
                    $default_services['interface'][$title]['monitor'] = 'off';
                }
                $default_services['interface'][$title]['warning'] = 10;
                $default_services['interface'][$title]['critical'] = 100;
                $default_services['interface'][$title]['name'] = $title;
            }

            // Create only one default process to monitor... we will add more via JS if someone wants to add more
            $default_services['process'][0]['monitor'] = 'off';
            $default_services['process'][0]['name'] = '';
            $default_services['process'][0]['display_name'] = '';
            $default_services['process'][0]['count']['warning'] = 60;
            $default_services['process'][0]['count']['critical'] = 100;

            // Create only one service too if there are no services saved
            $default_services['services'][0]['monitor'] = 'off';
            $default_services['services'][0]['name'] = '';
            $default_services['services'][0]['state'] = 'running';

            if (!isset($services)) {
                $services = $default_services;
            }

            $output .= '
            <h5 class="ul">' . _("CPU Metrics") . '</h5>
            <p>' . _("Specify the metrics you'd like to monitor on the NCPA Agent") . '.</p>
            <table class="table table-no-border table-auto-width table-padded">
                <tr>
                    <td class="vt">
                        <input type="checkbox" class="checkbox" name="services[cpu_usage][monitor]" id="cpum" ' . is_checked(grab_array_var($services['cpu_usage'], "monitor"), "on") . '>
                    </td>
                    <td>
                        <div>
                            <label for="cpum" style="line-height: auto;">
                                <b>' . _('CPU Usage') . '</b>
                            </label>
                            <div style="margin-bottom: 6px;">' . _('Check the CPU usage of the system') . '.</div>
                        </div>
                        <div>
                            <label><img src="'.theme_image('error.png').'" class="tt-bind" title="'._('Warning Threshold').'"></label>
                            <input type="text" size="1" name="services[cpu_usage][warning]" value="' . encode_form_val($services['cpu_usage']['warning']) . '" class="form-control condensed"> %&nbsp;&nbsp;
                            <label><img src="'.theme_image('critical_small.png').'" class="tt-bind" title="'._('Critical Threshold').'"></label>
                            <input type="text" size="1" name="services[cpu_usage][critical]" value="' . encode_form_val($services['cpu_usage']['critical']) . '" class="form-control condensed"> %
                        </div>
                    </td>
                </tr>
            </table>
            
            <h5 class="ul">' . _('Memory Metrics') . '</h5>
            <table class="table table-no-border table-auto-width table-padded">
                <tr>
                    <td class="vt">
                        <input type="checkbox" class="checkbox" name="services[memory_usage][monitor]" id="mainm" ' . is_checked(grab_array_var($services['memory_usage'], 'monitor'), 'on') . '>
                    </td>
                    <td>
                        <label class="normal" for="mainm">
                            <b>' . _('Main Memory Usage') . '</b><br>
                            ' . _('Monitor the main memory of the system. This metric is the percentage of main memory used') . '.
                        </label>
                        <div class="pad-t5">
                            <label><img src="'.theme_image('error.png').'" class="tt-bind" title="'._('Warning Threshold').'"></label>
                            <input type="text" size="1" name="services[memory_usage][warning]" value="' . encode_form_val($services['memory_usage']['warning']) . '" class="form-control condensed"> %&nbsp;&nbsp;
                            <label><img src="'.theme_image('critical_small.png').'" class="tt-bind" title="'._('Critical Threshold').'"></label>
                            <input type="text" size="1" name="services[memory_usage][critical]" value="' . encode_form_val($services['memory_usage']['critical']) . '" class="form-control condensed"> %
                        </div>
                    </td>
                </tr>
                <tr>
                    <td class="vt">
                        <input type="checkbox" class="checkbox" name="services[swap_usage][monitor]" id="swapm" ' . is_checked(grab_array_var($services['swap_usage'], 'monitor'), 'on') . '>
                    </td>
                    <td>
                        <label class="normal" for="swapm">
                            <b>' . _('Swap Usage') . '</b><br>
                            ' . _('Monitor the percentage of allocated swap used by the system') . '.
                        </label>
                        <div class="pad-t5">
                            <label><img src="'.theme_image('error.png').'" class="tt-bind" title="'._('Warning Threshold').'"></label>
                            <input type="text" size="1" name="services[swap_usage][warning]" value="' . encode_form_val($services['swap_usage']['warning']) . '" class="form-control condensed"> %&nbsp;&nbsp;
                            <label><img src="'.theme_image('critical_small.png').'" class="tt-bind" title="'._('Critical Threshold').'"></label>
                            <input type="text" size="1" name="services[swap_usage][critical]" value="' . encode_form_val($services['swap_usage']['critical']) . '" class="form-control condensed"> %
                        </div>
                    </td>
                </tr>
            </table>
    
            <h5 class="ul">' . _("Disk Metrics") . '</h5>
            <p>' . _("Specify the disks the the warning and critical percentages for disk capacity") . '.</p>
            <table class="table table-condensed table-no-border table-auto-width table-padded">';

            $id = 0;
            foreach ($services['disk'] as $title => $metrics) {
                $id++;
                $output .= '
                <tr>
                    <td>
                        <input type="checkbox" id="d' . $id . '" class="checkbox" name="services[disk][' . $title . '][monitor]" ' . is_checked(grab_array_var($services['disk'][$title], 'monitor'), 'on') . '>
                    </td>
                    <td>
                        <label for="d' . $id . '"><input type="text" class="form-control" name="services[disk][' . $title . '][name]" value="' . encode_form_val(str_replace("|", "/", $title)) . '" disabled></label>
                        <span style="margin-left: 6px;">
                            <label><img src="'.theme_image('error.png').'" class="tt-bind" title="'._('Warning Threshold').'"></label>
                            <input type="text" size="1" name="services[disk][' . $title . '][warning]" value="' . encode_form_val($services['disk'][$title]['warning']) . '" class="form-control condensed"> %&nbsp;&nbsp;
                            <label><img src="'.theme_image('critical_small.png').'" class="tt-bind" title="'._('Critical Threshold').'"></label>
                            <input type="text" size="1" name="services[disk][' . $title . '][critical]" value="' . encode_form_val($services['disk'][$title]['critical']) . '" class="form-control condensed"> %
                        </span>
                    </td>
                </tr>';
            }

            $output .= '
            </table>

            <h5 class="ul">' . _('Network Interface Metrics') . '</h5>
            <p>' . _("Specify reasonable bandwidths for your network interfaces. Note that these measurements are in megabits") . '.</p>
            <table class="table table-condensed table-no-border table-auto-width table-padded">';

            $id = 0;
            foreach ($services['interface'] as $title => $metrics) {
                $id++;
                $output .= '
                <tr>
                    <td>
                        <input type="checkbox" id="ni' . $id . '" class="checkbox" name="services[interface][' . $title . '][monitor]" ' . is_checked(grab_array_var($services['interface'][$title], 'monitor'), 'on') . '>
                    </td>
                    <td>
                        <label for="ni' . $id . '"><input type="text" class="form-control" style="width: 300px;" name="services[interface][' . $title . '][name]" value="' . encode_form_val(str_replace("|", "/", $title)) . '" disabled></label>
                        <span style="margin-left: 6px;">
                            <label><img src="'.theme_image('error.png').'" class="tt-bind" title="'._('Warning Threshold').'"></label>
                            <input type="text" size="2" name="services[interface][' . $title . '][warning]" value="' . encode_form_val($services['interface'][$title]['warning']) . '" class="form-control condensed"> Mb&nbsp;&nbsp;
                            <label><img src="'.theme_image('critical_small.png').'" class="tt-bind" title="'._('Critical Threshold').'"></label>
                            <input type="text" size="2" name="services[interface][' . $title . '][critical]" value="' . encode_form_val($services['interface'][$title]['critical']) . '" class="form-control condensed"> Mb
                        </span>
                    </td>
                </tr>';
            }

            $output .= '
            </table>

            <h5 class="ul">' . _("Services") . '</h5>
            <p>' . _("Specify which services should be running or stopped. Depending on the selected state you will recieve an OK when the process is in the selected state and a CRITICAL if the process is not in the state selected.") . '</p>
            <table class="table table-condensed table-no-border table-auto-width table-padded">
                <thead>
                    <tr>
                        <td></td>
                        <td></td>
                        <td>' . _("Service Name") . '</td>
                        <td>' . _("Expected Status") . '</td>
                    </tr>
                </thead>
                <tbody id="services-list">';

            foreach ($services['services'] as $i => $metrics) {
                $output .= '
                    <tr>
                        <td>
                            <input type="checkbox" name="services[services][' . $i . '][monitor]" ' . is_checked($metrics['monitor'], 'on') . '>
                        </td>
                        <td>
                            <span style="cursor: pointer;" class="service-selector" data-id="' . $i . '"><img src="' . theme_image('history2.png') . '" title="' . _("Select service from currently running services list") . '"></span>
                        </td>
                        <td>
                            <input type="text" class="form-control" style="width: 200px;" name="services[services][' . $i . '][name]" value="' . encode_form_val($metrics['name']) . '">
                        </td>
                        <td>
                            <label class="normal" style="margin-right: 6px;"><input name="services[services][' . $i . '][state]" style="vertical-align: text-bottom;" type="radio" value="running" ' . is_checked($metrics['state'], 'running') . '> ' . _("Running") . '</label><label class="normal"><input type="radio" style="vertical-align: text-bottom;" value="stopped" name="services[services][' . $i . '][state]" ' . is_checked($metrics['state'], 'stopped') . '> ' . _("Stopped") . '</label>
                        </td>
                    </tr>';
            }

            // Create a list of services for the JS
            $service_list = '';
            ksort($root['services']);
            foreach ($root['services'] as $service => $status) {
                $service_list .= '<option value="' . $service . '" data-status="' . $status . '">' . $service . ' (' . $status . ')</option>';
            }

            $output .= '
                </tbody>
            </table>
            <div style="margin: 10px 0 20px 0;">
                <a style="cursor: pointer;" id="add-new-service">Add Another Service Check</a>
            </div>
    
            <h5 class="ul">' . _('Running Processes') . '</h5>
            <p>' . _("Specify which processes should be running, and how many should be") . '.</p>
            <table class="table table-condensed table-no-border table-auto-width table-padded">
                <thead>
                    <tr>
                        <td></td>
                        <td><label>Process Name</label></td>
                        <td><label>Display Name</label></td>
                        <td><label>Warning #</label></td>
                        <td><label>Critical #</label></td>
                    </tr>
                </thead>
                <tbody id="process-list">';

            foreach ($services['process'] as $i => $metrics) {
                $output .= '
                    <tr>
                        <td>
                            <input type="checkbox" class="checkbox deselect" name="services[process][' . $i . '][monitor]" ' . is_checked($metrics['monitor'], 'on') . '>
                        </td>
                        <td style="padding-right: 10px;">
                            <input type="text" class="destext form-control" name="services[process][' . $i . '][name]" value="' . encode_form_val($metrics['name']) . '">
                        </td>
                        <td style="padding-right: 10px;">
                            <input type="text" class="destext form-control" name="services[process][' . $i . '][display_name]" value="' . encode_form_val($metrics['display_name']) . '">
                        </td>
                        <td>
                            <input type="text" class="destext form-control" size="2" name="services[process][' . $i . '][count][warning]" value="' . encode_form_val($metrics['count']['warning']) . '">
                        </td>
                        <td>
                            <input type="text" class="destext form-control" size="2" name="services[process][' . $i . '][count][critical]" value="' . encode_form_val($metrics['count']['critical']) . '">
                        </td>
                    </tr>';
            }

            $output .= '
                </tbody>
            </table>
            <div style="margin: 10px 0 20px 0;">
                <a style="cursor: pointer;" id="add-new-process">Add Another Process Check</a>
            </div>
            <script type="text/javascript">
            var processnum = ' . (count($services['process']) - 1) . ';
            var servicenum = ' . (count($services['services']) - 1) . ';
            $(document).ready(function() {

                $("#add-new-process").click(function() {
                    processnum++;
                    $("#process-list").append(\'<tr><td><input type="checkbox" class="checkbox deselect" name="services[process][\'+processnum+\'][monitor]" /></td><td style="padding-right: 10px;"><input type="text" class="destext form-control" name="services[process][\'+processnum+\'][name]" value=""></td><td style="padding-right: 10px;"><input type="text" class="destext form-control" name="services[process][\'+processnum+\'][display_name]" value=""></td><td><input type="text" class="destext form-control" size="2" name="services[process][\'+processnum+\'][count][warning]" value="60"></td><td><input type="text" class="destext form-control" size="2" name="services[process][\'+processnum+\'][count][critical]" value="100"></td></tr>\');
                });

                $("#add-new-service").click(function() {
                    servicenum++;
                    $("#services-list").append(\'<tr><td><input type="checkbox" name="services[services][\'+servicenum+\'][monitor]"></td><td><span style="cursor: pointer;" class="service-selector" data-id="\'+servicenum+\'"><img src="' . theme_image('history2.png') . '" title="' . _("Select service from currently running services list") . '"></span></td><td><input type="text" class="form-control" style="width: 200px;" name="services[services][\'+servicenum+\'][name]" value=""></td><td><label class="normal" style="margin-right: 6px;"><input name="services[services][\'+servicenum+\'][state]" type="radio" value="running" style="vertical-align: text-bottom;" checked> ' . _("Running") . '</label><label class="normal"><input type="radio" value="stopped" style="vertical-align: text-bottom;" name="services[services][\'+servicenum+\'][state]"> ' . _("Stopped") . '</label></td></tr>\');
                });

                $("#services-list").on("click", "span.service-selector", function() {
                    var service_id = $(this).data("id");
                    var content = \'<div><h2>' . _("Services listed by the NCPA Agent") . '</h2><p>' . _("Select a service that is either running or stopped from the NCPA client host to atuomatically fill in the service name and the expected state.") . '</p><div><select id="selected-service" class="form-control">' . $service_list . '</select></div><div style="margin-top: 6px;"><button data-serviceid="\'+service_id+\'" class="btn btn-sm btn-primary" id="add-selected-service">' . _("Select this Service") . '</button></div></div>\';
                    
                    $("#child_popup_container").width(450);
                    $("#child_popup_layer").width(480);
                    $("#child_popup_layer").css("position", "fixed");
                    set_child_popup_content(content);
                    display_child_popup();
                });

                $("#child_popup_container").on("click", "#add-selected-service", function() {
                    var service_id = $(this).data("serviceid");
                    var selected_service = $("#selected-service option:selected").val();
                    var selected_state = $("#selected-service option:selected").data("status");
                    $(\'input[name="services[services][\'+service_id+\'][name]"]\').val(selected_service);
                    $(\'input[name="services[services][\'+service_id+\'][state]"][value="\'+selected_state+\'"]\').prop("checked", true);
                    close_child_popup();
                });

            });
            </script>
            ';
            break;

        case CONFIGWIZARD_MODE_VALIDATESTAGE2DATA:
            // Get variables that were passed to us
            $address = grab_array_var($inargs, 'address');
            $hostname = grab_array_var($inargs, 'hostname');
            $port = grab_array_var($inargs, 'port');
            $token = grab_array_var($inargs, 'token');

            // Check for errors
            $errors = 0;
            $errmsg = array();
            if (is_valid_host_name($hostname) == false) {
                $errmsg[$errors++] = "Invalid host name.";
            }

            if ($errors > 0) {
                $outargs[CONFIGWIZARD_ERROR_MESSAGES] = $errmsg;
                $result = 1;
            }
            break;

        case CONFIGWIZARD_MODE_GETSTAGE3HTML:
            // Get variables that were passed to us
            $address = grab_array_var($inargs, 'address');
            $hostname = grab_array_var($inargs, 'hostname');
            $port = grab_array_var($inargs, 'port');
            $token = grab_array_var($inargs, 'token');
            $services = grab_array_var($inargs, 'services', array());

            $output = '
            <input type="hidden" name="address" value="' . encode_form_val($address) . '" />
            <input type="hidden" name="hostname" value="' . encode_form_val($hostname) . '" />
            <input type="hidden" name="port" value="' . encode_form_val($port) . '" />
            <input type="hidden" name="token" value="' . encode_form_val($token) . '" />
            <input type="hidden" name="services_serial" value="' . base64_encode(serialize($services)) . '" />';
            break;

        case CONFIGWIZARD_MODE_VALIDATESTAGE3DATA:
            break;

        case CONFIGWIZARD_MODE_GETFINALSTAGEHTML:
            $output = '';
            break;

        case CONFIGWIZARD_MODE_GETOBJECTS:
            $hostname = grab_array_var($inargs, "hostname", "");
            $address = grab_array_var($inargs, "address", "");
            $hostaddress = $address;
            $port = grab_array_var($inargs, "port", "");
            $token = grab_array_var($inargs, "token", "");
            $services_serial = grab_array_var($inargs, "services_serial", "");
            $services = unserialize(base64_decode($services_serial));

            // Save data for later use in re-entrance
            $meta_arr = array();
            $meta_arr["hostname"] = $hostname;
            $meta_arr["address"] = $address;
            $meta_arr["port"] = $port;
            $meta_arr["token"] = $token;
            $meta_arr["services"] = $services;
            save_configwizard_object_meta($wizard_name, $hostname, "", $meta_arr);

            $objs = array();
            if (!host_exists($hostname)) {
                $objs[] = array(
                    "type" => OBJECTTYPE_HOST,
                    "use" => "xiwizard_ncpa_host",
                    "host_name" => $hostname,
                    "address" => $hostaddress,
                    "icon_image" => "ncpa.png",
                    "statusmap_image" => "ncpa.png",
                    "_xiwizard" => $wizard_name);
            }

            // Common plugin opts
            $commonopts = "-t '$token' ";
            if ($port) {
                $commonopts .= "-P $port ";
            }

            foreach ($services as $type => $args) {
                $pluginopts = "";
                $pluginopts .= $commonopts;

                switch ($type) {

                    case "cpu_usage":
                        if (!array_key_exists('monitor', $args)) {
                            break;
                        }
                        $pluginopts .= "-M cpu/percent";

                        if (!empty($args['warning'])) {
                            $pluginopts .= " -w " . $args["warning"];
                        }
                        if (!empty($args['critical'])) {
                            $pluginopts .= " -c " . $args["critical"];
                        }

                        $objs[] = array(
                            "type" => OBJECTTYPE_SERVICE,
                            "host_name" => $hostname,
                            "service_description" => "CPU Usage",
                            "use" => "xiwizard_ncpa_service",
                            "check_command" => "check_xi_ncpa_agent!" . $pluginopts,
                            "_xiwizard" => $wizard_name);
                        break;

                    case "memory_usage":
                        if (!array_key_exists('monitor', $args)) {
                            break;
                        }
                        $pluginopts .= "-M memory/virtual/percent";

                        if (!empty($args['warning'])) {
                            $pluginopts .= " -w " . $args["warning"];
                        }
                        if (!empty($args['critical'])) {
                            $pluginopts .= " -c " . $args["critical"];
                        }

                        $objs[] = array(
                            "type" => OBJECTTYPE_SERVICE,
                            "host_name" => $hostname,
                            "service_description" => "Main Memory Usage",
                            "use" => "xiwizard_ncpa_service",
                            "check_command" => "check_xi_ncpa_agent!" . $pluginopts,
                            "_xiwizard" => $wizard_name);
                        break;

                    case "swap_usage":
                        if (!array_key_exists('monitor', $args)) {
                            break;
                        }
                        $pluginopts .= "-M memory/swap/percent";

                        if (!empty($args['warning'])) {
                            $pluginopts .= " -w " . $args["warning"];
                        }
                        if (!empty($args['critical'])) {
                            $pluginopts .= " -c " . $args["critical"];
                        }

                        $objs[] = array(
                            "type" => OBJECTTYPE_SERVICE,
                            "host_name" => $hostname,
                            "service_description" => "Swap Usage",
                            "use" => "xiwizard_ncpa_service",
                            "check_command" => "check_xi_ncpa_agent!" . $pluginopts,
                            "_xiwizard" => $wizard_name);
                        break;

                    case "disk":
                        foreach ($args as $title => $metrics) {
                            if (!array_key_exists('monitor', $metrics)) {
                                continue;
                            }
                            $theseopts = "{$pluginopts} -M 'disk/logical/{$title}/used_percent'";

                            if (!empty($metrics["warning"])) {
                                $theseopts .= " -w " . $metrics["warning"];
                            }
                            if (!empty($metrics["critical"])) {
                                $theseopts .= " -c " . $metrics["critical"];
                            }

                            $objs[] = array(
                                "type" => OBJECTTYPE_SERVICE,
                                "host_name" => $hostname,
                                "service_description" => "Disk Usage on " . $title,
                                "use" => "xiwizard_ncpa_service",
                                "check_command" => "check_xi_ncpa_agent!" . $theseopts,
                                "_xiwizard" => $wizard_name);
                        }
                        break;

                    case "interface":
                        foreach ($args as $title => $metrics) {
                            if (!array_key_exists('monitor', $metrics)) {
                                continue;
                            }

                            $theseopts = "{$pluginopts} -M 'interface/{$title}/bytes_sent' -d -u M";

                            if (!empty($metrics["warning"])) {
                                $theseopts .= " -w " . $metrics["warning"];
                            }
                            if (!empty($metrics["critical"])) {
                                $theseopts .= " -c " . $metrics["critical"];
                            }

                            $objs[] = array(
                                "type" => OBJECTTYPE_SERVICE,
                                "host_name" => $hostname,
                                "service_description" => "Outbound Bandwidth on {$title}",
                                "use" => "xiwizard_ncpa_service",
                                "check_command" => "check_xi_ncpa_agent!" . $theseopts,
                                "_xiwizard" => $wizard_name);

                            $theseopts = "{$pluginopts} -M 'interface/{$title}/bytes_recv' -d -u M";

                            if (!empty($metrics["warning"])) {
                                $theseopts .= " -w " . $metrics["warning"];
                            }
                            if (!empty($metrics["critical"])) {
                                $theseopts .= " -c " . $metrics["critical"];
                            }

                            $objs[] = array(
                                "type" => OBJECTTYPE_SERVICE,
                                "host_name" => $hostname,
                                "service_description" => "Inbound Bandwidth on {$title}",
                                "use" => "xiwizard_ncpa_service",
                                "check_command" => "check_xi_ncpa_agent!" . $theseopts,
                                "_xiwizard" => $wizard_name);
                        }
                        break;

                    case "services":
                        foreach ($args as $i => $service) {
                            if (!array_key_exists('monitor', $service)) {
                                continue;
                            }
                            $theseopts = "{$pluginopts} -M 'service/" . $service["name"] . "/" . $service["state"] . "'";
                            $objs[] = array(
                                "type" => OBJECTTYPE_SERVICE,
                                "host_name" => $hostname,
                                "service_description" => "Service Status: " . $service["name"],
                                "use" => "xiwizard_ncpa_service",
                                "check_command" => "check_xi_ncpa_agent!" . $theseopts,
                                "_xiwizard" => $wizard_name);
                        }
                        break;

                    case "process":
                        foreach ($args as $i => $metrics) {
                            if (!array_key_exists('monitor', $metrics)) {
                                continue;
                            }
                            $proc_name = $metrics['name'];
                            $display = $metrics['display_name'];
                            $theseopts = "{$pluginopts} -M 'process/{$proc_name}/count'";

                            if (!empty($metrics["count"]["warning"])) {
                                $theseopts .= " -w " . $metrics["count"]["warning"];
                            }
                            if (!empty($metrics["count"]["critical"])) {
                                $theseopts .= " -c " . $metrics["count"]["critical"];
                            }

                            $objs[] = array(
                                "type" => OBJECTTYPE_SERVICE,
                                "host_name" => $hostname,
                                "service_description" => "Instances of {$display}",
                                "use" => "xiwizard_ncpa_service",
                                "check_command" => "check_xi_ncpa_agent!" . $theseopts,
                                "_xiwizard" => $wizard_name);
                        }
                        break;

                    default:
                        break;
                }
            }

            // Return the object definitions to the wizard
            $outargs[CONFIGWIZARD_NAGIOS_OBJECTS] = $objs;
            break;

        default:
            break;
    }

    return $output;
}